% DRYAD simulation
% Code started 06/05, last revised 06/11
% Aaron Benjamin, University of Illinois at Urbana-Champaign

% The model implemented here is described in:
%   Benjamin, A. S. (2011).  Representational explanations of process 
%   dissociations in recognition: The DRYAD theory of aging and memory
%   judgments.

% Please refer to that article if you use this code or implement this model.

% This code is not at all optimized to run quickly.  I welcome all
% revisions of it.  It has been written in Matlab version 7.8.0 (R2009a).

% This version of the code performs the exclusion task with two contexts.

% Clear variables and set randomizer seed
clc;
clear;
clk = clock;
rand('seed',clk(6));

% Simulation parameters

% Maximum and minimum correlation between the two context vectors
% The program can grind to a halt if these are set to oconservatively.
maxR = .8;
minR = -0.8;

% Starting, ending, and step-size values for the decision criterion
CriterionBegin = 0.05;
CriterionEnd = 0.05;
CriterionStep = 0.05;

% Total number of study items (should be divisible by 2, for 2 contexts)
NumStudyItems = 40;

% Number of different contexts (code is set to work for 2)
NumContexts = 2;

% Number of (new) distractors on the exclusion test
NumDistract = 40;

% Number of sim-subjects for EACH set of parameter values.
NumSubjects = 100;

% Starting, ending, and step-size value for the memory fidelity factor (L)
% (This parameter is called F in the paper.)
Lbegin = .1;
Lend = .9;
Lstep = .1;
Lcount = 0;

% Starting, ending, and step-size value for the number of dimensions
% accorded to the representation of the 'ITEM' (In).
InodeBegin=20;
InodeEnd=20;
InodeStep=5;

% Starting, ending, and step-size value for the number of dimensions
% accorded to the representation of the 'CONTEXT' (Cn).
CnodeBegin=2;
CnodeEnd=6;
CnodeStep=2;

% These parameters are defined so the progress bar will display.
Rcount=0;
NumCritLoops =((CriterionEnd-CriterionBegin)/CriterionStep)+1;
NumILoops=((InodeEnd-InodeBegin)/InodeStep)+1;
NumCLoops=((CnodeEnd-CnodeBegin)/CnodeStep)+1;
NumLLoops=((Lend-Lbegin)/Lstep)+1;
TotalLoops=NumILoops*NumCLoops*NumLLoops*NumCritLoops*NumSubjects;
h = waitbar(0,'Simulating...');

% Simulation starts here
for Inodes = InodeBegin:InodeStep:InodeEnd
for Cnodes = CnodeBegin:CnodeStep:CnodeEnd
for L=Lbegin:Lstep:Lend
for criterion = CriterionBegin:CriterionStep:CriterionEnd
    Lcount = Lcount +1;
for subject=1:NumSubjects
    Rcount=Rcount+1;
    progress = ((subject+((Lcount-1)*NumSubjects))/TotalLoops);
    waitbar(progress);
    
    % Create to-be-learned stimuli
    [stimuli contexts] = create_stimuli(Inodes,Cnodes,NumStudyItems,NumContexts,minR,maxR);
    
    % Encode stimuli into the model
    memory = learn(stimuli, L);
    
    % Create test list (add new distractors to studied items and remove
    % contexts)
    test = create_test(stimuli,NumDistract,Inodes);
    
    % Compute familiarity (called Mg in the paper) and compare to criterion
    [A, I] = Icompute(memory, test);
    F = evaluate_familiarity(I, criterion);
    
    % Recall (called *e* in the paper) and choose the most likely context
    C = recall_context(memory, test, A, contexts);
    [S, DP] = choose_contexts(C, contexts, NumContexts, Cnodes);
    
    % Combine fanmiliarity and recollected context into a decision
    [R] = response(S, F);
    
    % Generate HR, FAR-TBX, and FAR-new for this subject, and place the values into matrix D.
    [HR FARtbx FARnew FARnew_F] = compute_parameters(F, R, NumStudyItems, NumDistract, NumContexts, subject);
    clear stimuli contexts memory test A I F C contexts S DP R;
    D(subject,1) = HR;
    D(subject,2) = FARtbx;
    D(subject,3) = FARnew;
    clear HR FARtbx FARnew FARnew_F;
end;

% After running all NumSubjects for a given set of parameters, encode into
% matrix D the relevant parameters (Inodes, Cnodes, L, and criterion) and
% experimental statistics (mean and variance of HR, FAR-TBX, and FAR-new).
%
% This creates a 10 column tables of the following format:
% 1:Inodes | 2:Cnodes | 3:L | 4:Criterion | 5:Mean HR
% 6:Mean FAR-TBX | 7:Mean FAR-new | 8:Var HR | 9:Var FAR-TBX | 10:Var FAR-new
%
% M can be cut and pasted into Excel for easy graphing and analysis.

M(Lcount,1)=Inodes;
M(Lcount,2)=Cnodes;
M(Lcount,3)=L;
M(Lcount,4)=criterion;
M(Lcount,5:7)=mean(D);
M(Lcount,8:10)=var(D);
end;
end;
end;
end;
close(h);
